import java.awt.Point;

class EnsemblePoints {
  /*------------------- Champs ------------------------*/
  // l'ensemble des points
  Point [] points ;
  
  /*------------------- Constructeurs ------------------------*/
  /************************************
   * Creer un tableau de points dont les coordonnées sont sur l'écran 
   *
   * entrée :
   *   - n : nombre de points 
   *
   * sortie :
   *   - aucune
   *************************************/
  EnsemblePoints(int n) {
    points = new Point[n];

    for (int i = 0; i< n; i++) {
      points[i] = new Point(int(random(width)), int(random(height)));
    }
  }

 
  /*------------------- Méthodes ------------------------*/
  /************************************
   * dessine les points contenus dans le tableau 
   *
   * entrée :
   *   - aucune  
   *
   * sortie :
   *   - aucune seulement sortie graphique
   *************************************/
  void dessine() {
    for (int i = 0; i < points.length; i++)
      point( (float) points[i].getX(), (float) points[i].getY());
  }

  /****************************************
   * calcule la distance entre les points
   *
   * entree :
   *   - x1, y1 : coordonée du point 1
   *   - x2, y2 : coordonée du point 2
   *
   * sortie :
   *   - distance
   *****************************************/
  float distance(float x1, float y1, float x2, float y2) {
    return sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
  }

  /************************************
   *  dénombre les points à l'intérieur d'un cercle
   *
   * entrée :
   *   - x,y : coordonnée du centre du cercle
   *   - r : rayon du cercle 
   *
   * sortie :
   *   - nombre de points
   *************************************/
  int nbCercle(float x, float y, float r) {
    int nb = 0;

    for (int i = 0; i < points.length; i++) {
      if (distance(x, y, (float) points[i].getX(), (float) points[i].getY()) < r)
        nb++;
    }

    return nb;
  }

  /************************************
   *  détermine le point le plus proche du cercle
   *
   * entrée :
   *   - x,y : coordonnée du centre du cercle
   *
   * sortie :
   *   - point le plus proche
   *************************************/
  Point plusProche(float x, float y) {
    Point p = points[0];

    for (int i = 1; i < points.length; i++) {
      if (distance(x, y, (float) points[i].getX(), (float) points[i].getY()) 
        < distance(x, y, (float) p.getX(), (float) p.getY()))
        p = points[i];
    }

    return p;
  }
}

void setup() {
  size(300, 300);

  EnsemblePoints points = new EnsemblePoints(200);

  float xc = width  / 2;
  float yc = height / 2;
  float r = 100;

  fill(255, 0, 0);
  ellipse(xc, yc, 2 * r, 2 * r);

  strokeWeight(10);
  points.dessine();

  println("nombre de points dans le cercle : " + points.nbCercle(xc, yc, r));

  println("point le plus proche du centre du cercle en bleu");

  Point p = points.plusProche(xc, yc);
  stroke(0, 0, 255);
  point((float) p.getX(), (float) p.getY());
}

